﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Serialization;
using System.Diagnostics;
using System.Text.RegularExpressions;
using System.IO;
using DirectShowLib;
using System.Runtime.InteropServices;

namespace KLToolBox
{
    public class Configuration
    {
        int _Version = 1;
        string _rcTVCapLocation;
        string _FileDumpFilter;
        string _CaptureLocation;
        string _GraphFile;
        string _DGAVCDec;
        string _tsMuxer;
        string _ChannelChange;
        string _TemplatePath;
        string _x264Path;
        string _x264Template;
        string _CCParam;
        string _NodOff;
        string _NodOffTime;

        public static void Serialize(string file, Configuration c)
        {
            System.Xml.Serialization.XmlSerializer xs =
                new System.Xml.Serialization.XmlSerializer(c.GetType());
            StreamWriter writer = File.CreateText(file);
            xs.Serialize(writer, c);
            writer.Flush();
            writer.Close();
        }
        public static Configuration Deserialize(string file)
        {
            if (!File.Exists(file))
            {
                MessageBox.Show("Could not find config.xml\r\n Use Save Config\r\n after Setting Locations in Program");
                Configuration x = new Configuration();
                Serialize(file, x);
                return (x);
            }
//            MessageBox.Show("Found config.xml" + file);
            System.Xml.Serialization.XmlSerializer xs
                = new System.Xml.Serialization.XmlSerializer(
                    typeof(Configuration));
            StreamReader reader = File.OpenText(file);
            Configuration c = (Configuration)xs.Deserialize(reader);
            reader.Close();
//            MessageBox.Show(c.rcTVCapLocation + "\r\n" + c.FileDumpFilter + "\r\n" + c.CaptureLocation + "\r\n" + c._x264Template + "\r\n");
            return c;
        }
        public int Version
        {
            get { return _Version; }
            set { _Version = value; }
        }
        public string rcTVCapLocation
        {
            get { return _rcTVCapLocation; }
            set { _rcTVCapLocation = value; }
        }
        public string FileDumpFilter
        {
            get { return _FileDumpFilter; }
            set { _FileDumpFilter = value; }
        }
        public string CaptureLocation
        {
            get { return _CaptureLocation; }
            set { _CaptureLocation = value; }
        }
        public string GraphFile
        {
            get { return _GraphFile; }
            set { _GraphFile = value; }
        }
        public string DGAVCDec
        {
            get { return _DGAVCDec; }
            set { _DGAVCDec = value; }
        }
        public string tsMuxer
        {
            get { return _tsMuxer; }
            set { _tsMuxer = value; }
        }
        public string ChannelChange
        {
            get { return _ChannelChange; }
            set { _ChannelChange = value; }
        }
        public string TemplatePath
        {
            get { return _TemplatePath; }
            set { _TemplatePath = value; }
        }
        public string x264Path
        {
            get { return _x264Path; }
            set { _x264Path = value; }
        }
        public string x264Template
        {
            get { return _x264Template; }
            set { _x264Template = value; }
        }
        public string CCParam
        {
            get { return _CCParam; }
            set { _CCParam = value; }
        }
        public string NodOff
        {
            get { return _NodOff; }
            set { _NodOff = value; }
        }
        public string NodOffTime
        {
            get { return _NodOffTime; }
            set { _NodOffTime = value; }
        }
    }

    public class Dirt
    {
        public int dirt;
    }
    public class FilterProfiles
    {
        public const String XMLName = "FilterProfiles.xml";

        public const string HDPVRFilterName = "Hauppauge HD PVR Encoder";
        public const string HDPVRAvgBitrate = "49cc4c43-ca83-4ad4-a9af-f3696af666df";
        public const string HDPVRPeakBitrate = "703f16a9-3d48-44a1-b077-018dff915d19";
        public const string HDPVRBitrateMode = "ee5fb25c-c713-40d1-9d58-c0d7241e250f";

        public class Profile
        {
            public class PropSet
            {
                public string Comment = "";
                public string FilterName = "";
                public string Guid = "";
                public int ID = 0;
                public int Value = 0;
            }
            public string Name = "";
            public string Comment = "";
            public List<PropSet> FilterProps = new List<PropSet>();
        }

        public List<Profile> Profiles = new List<Profile>();

        public static Profile CreateProfile(int avgbitrate, int peakbitrate, int mode)
        {
            Profile p = new Profile();
            // Create Average Bitrate PropSet
            Profile.PropSet ps = new Profile.PropSet();
            ps.Comment = "Average Bitrate";
            ps.FilterName = HDPVRFilterName;
            ps.Guid = HDPVRAvgBitrate;
            ps.ID = 0;
            ps.Value = avgbitrate;
            p.FilterProps.Add(ps);

            // Create Peak Bitrate PropSet
            ps = new Profile.PropSet();
            ps.Comment = "Peak Bitrate";
            ps.FilterName = HDPVRFilterName;
            ps.Guid = HDPVRPeakBitrate;
            ps.ID = 0;
            ps.Value = peakbitrate;
            p.FilterProps.Add(ps);

            // Create Mode PropSet
            ps = new Profile.PropSet();
            ps.Comment = "Bitrate Mode";
            ps.FilterName = HDPVRFilterName;
            ps.Guid = HDPVRBitrateMode;
            ps.ID = 0;
            ps.Value = mode;
            p.FilterProps.Add(ps);

            return (p);
        }

        public static object LoadFromFileFast(System.Type derivedType, string aFullFile)
        {
            //System.Type derivedType = aObj.GetType();
            object ret = null;
            try
            {

                if (File.Exists(aFullFile))
                {
                    try
                    {
                        XmlSerializer ser = new XmlSerializer(derivedType);
                        using (FileStream stream = File.OpenRead(aFullFile))
                        {
                            ret = ser.Deserialize(stream);
                        }
                    }
                    catch (Exception ex)
                    {
                             throw new Exception("LoadFromFileFast failed. tried too many times: " + aFullFile);
                    }
                }
                else
                {
                    throw new Exception("file does not exist: " + aFullFile);
                }
                return ret;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
    }
}