﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;

namespace KLToolBox
{
    /// <summary>
    /// Parse settings from ini-like files
    /// </summary>
    public class IniFile
    {
        static IniFile()
        {
            _iniKeyValuePatternRegex = new Regex(
            @"((\s)*(?<Key>([^\=^\s^\n]+))[\s^\n]*
# key part (surrounding whitespace stripped)
\=
(\s)*(?<Value>([^\n]+(\n){0,1})))
# value part (surrounding whitespace stripped)
",
            RegexOptions.IgnorePatternWhitespace |
            RegexOptions.Compiled |
            RegexOptions.CultureInvariant);
        }
        static private Regex _iniKeyValuePatternRegex;

        public IniFile(string iniFileName)
        {
            _iniFileName = iniFileName;
        }

        public string ParseFileReadValue(string key)
        {
            using (StreamReader reader =
            new StreamReader(_iniFileName))
            {
                do
                {
                    string line = reader.ReadLine();
                    Match match =
                    _iniKeyValuePatternRegex.Match(line);
                    if (match.Success)
                    {
                        string currentKey =
                        match.Groups["Key"].Value as string;
                        if (currentKey != null &&
                        currentKey.Trim().CompareTo(key) == 0)
                        {
                            string value =
                            match.Groups["Value"].Value as string;
                            return value;
                        }
                    }

                }
                while (reader.Peek() != -1);
            }
            return null;
        }

        public string IniFileName
        {
            get { return _iniFileName; }
        } private string _iniFileName;
    }
}